/*
** File: interrupt.c  
** Project: Linux Driver Framework
** Purpose: Interrupt hooking functions
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"
#include "dflinux.h"

#if (LINUX_VERSION_CODE >= 0x020613)
irqreturn_t
_dfLinuxIsrStub(
  int irq,
  void* pContext)
#else
irqreturn_t
_dfLinuxIsrStub(
  int irq,
  void* pContext,
  struct pt_regs* pPtRegs /* ignored */)
#endif
{
  DfInterruptObject* pInterruptObject = (DfInterruptObject*)pContext;
  boolean_t bHandled;
  DfSpinLockFlags f;

  dfAssert(pInterruptObject->bConnected);

  f = dfSpinLockGet(&pInterruptObject->lock);
  bHandled = pInterruptObject->pIsr(pInterruptObject, pInterruptObject->pContext);
  dfSpinLockPut(&pInterruptObject->lock, f);

  return bHandled ? IRQ_HANDLED : IRQ_NONE;
}

boolean_t
dfInterruptConnect(
  DfDeviceObject* pDevObj,
  DfInterruptObject* pInterruptObject,
  DfBusResources* pBusResources,
  DfBusType busType,
  unsigned int interruptIndex,
  DfIsr* pIsr,
  void* pContext)
{
  int res, vector;
  unsigned long flags;

  dfAssert(busType == DfBusPci);
  dfAssert(!pInterruptObject->bConnected);
  dfAssert(interruptIndex < pBusResources->variant.pci.numIrq);

#if (LINUX_VERSION_CODE >= 0x020612)
  flags = IRQF_SHARED;
#else
  /* SA_INTERRUPT => fast handler, so interrupts are disabled in ISR */
  flags = SA_SHIRQ | SA_INTERRUPT;
#endif

  dfSpinLockInit(&pInterruptObject->lock);
  pInterruptObject->pIsr = pIsr;
  pInterruptObject->pContext = pContext;
  vector = pBusResources->variant.pci.irq[interruptIndex].variant.translated.system.vector;
  pInterruptObject->vector = vector;
  pInterruptObject->bConnected = TRUE;
  res = request_irq(vector, _dfLinuxIsrStub, flags, "adb3", pInterruptObject);
  if (res) {
    pInterruptObject->bConnected = FALSE;
    return FALSE;
  }
  return TRUE;
}

void
dfInterruptDisconnect(
  DfDeviceObject* pDevObj,
  DfInterruptObject* pInterruptObject)
{
  dfAssert(pInterruptObject->bConnected);
  free_irq(pInterruptObject->vector, pInterruptObject);
}
